module ddr3_rw(
	ui_clk,
	ui_clk_sync_rst,
	init_calib_complete,
	app_rdy,
	app_wdf_rdy,
	app_rd_data_valid,
	wfifo_rcount,
	rfifo_wcount,
	rd_load,
	wr_load,
	app_addr_rd_min,
	app_addr_rd_max,
	rd_bust_len,
	app_addr_wr_min,
	app_addr_wr_max,
	wr_bust_len,
	ddr3_read_valid,
	ddr3_pingpang_en,
	rfifo_wren,
	app_addr,
	app_en,
	app_wdf_wren,
	app_wdf_end,
	app_cmd);

	//---Ports declearation: generated by Robei---
	input ui_clk;
	input ui_clk_sync_rst;
	input init_calib_complete;
	input app_rdy;
	input app_wdf_rdy;
	input app_rd_data_valid;
	input [10:0] wfifo_rcount;
	input [10:0] rfifo_wcount;
	input rd_load;
	input wr_load;
	input [27:0] app_addr_rd_min;
	input [27:0] app_addr_rd_max;
	input [7:0] rd_bust_len;
	input [27:0] app_addr_wr_min;
	input [27:0] app_addr_wr_max;
	input [7:0] wr_bust_len;
	input ddr3_read_valid;
	input ddr3_pingpang_en;
	output rfifo_wren;
	output [27:0] app_addr;
	output app_en;
	output app_wdf_wren;
	output app_wdf_end;
	output [2:0] app_cmd;

	wire ui_clk;
	wire ui_clk_sync_rst;
	wire init_calib_complete;
	wire app_rdy;
	wire app_wdf_rdy;
	wire app_rd_data_valid;
	wire [10:0] wfifo_rcount;
	wire [10:0] rfifo_wcount;
	wire rd_load;
	wire wr_load;
	wire [27:0] app_addr_rd_min;
	wire [27:0] app_addr_rd_max;
	wire [7:0] rd_bust_len;
	wire [27:0] app_addr_wr_min;
	wire [27:0] app_addr_wr_max;
	wire [7:0] wr_bust_len;
	wire ddr3_read_valid;
	wire ddr3_pingpang_en;
	wire rfifo_wren;
	reg [27:0] app_addr;
	wire app_en;
	wire app_wdf_wren;
	wire app_wdf_end;
	wire [2:0] app_cmd;

	//----Code starts here: integrated by Robei-----
	//localparam 
	localparam IDLE        = 4'b0001;   //״̬
	localparam DDR3_DONE   = 4'b0010;   //DDR3ʼ״̬
	localparam WRITE       = 4'b0100;   //FIFO״̬
	localparam READ        = 4'b1000;   //дFIFO״̬
	
	//reg define
	//reg    [27:0] app_addr;             DDR3ַ 
	reg    [27:0] app_addr_rd;          //DDR3ַ
	reg    [27:0] app_addr_wr;          //DDR3дַ
	reg    [3:0]  state_cnt;            //״̬
	reg    [23:0] rd_addr_cnt;          //ûַ
	reg    [23:0] wr_addr_cnt;          //ûдַ 
	reg    [8:0]  burst_rd_cnt;         //ͻ
	reg    [8:0]  burst_wr_cnt;         //ͻд    
	reg    [10:0] raddr_rst_h_cnt;      //Դ֡λм 
	reg    [27:0] app_addr_rd_min_a;    //DDR3ʼַ
	reg    [27:0] app_addr_rd_max_a;    //DDR3Ľַ
	reg    [7:0]  rd_bust_len_a;        //DDR3жʱͻ
	reg    [27:0] app_addr_wr_min_a;    //дDDR3ʼַ
	reg    [27:0] app_addr_wr_max_a;    //дDDR3Ľַ
	reg    [7:0]  wr_bust_len_a;        //DDR3дʱͻ
	reg           star_rd_flag;         //λд2֡ı־ź
	reg           rd_load_d0;
	reg           rd_load_d1;
	reg           raddr_rst_h;          //Դ֡λ
	reg           wr_load_d0;
	reg           wr_load_d1;
	reg           wr_rst;               //Դ֡λ־
	reg           rd_rst;               //Դ֡λ־
	reg           raddr_page;           //ddr3ַлź
	reg           waddr_page;           //ddr3дַлź
	reg           burst_done_wr;        //һͻдź
	reg           burst_done_rd;        //һζдź
	reg           wr_end;               //һͻдź
	reg           rd_end;               //һζдź   
	
	wire          rst_n;
	
	
	//ЧźŸwfifoдʹ
	assign rfifo_wren =  app_rd_data_valid;
	
	assign rst_n = ~ui_clk_sync_rst;
	
	//д״̬MIGдЧ,ڶ״̬MIGУʱʹźΪߣΪ
	assign app_en = ((state_cnt == WRITE && (app_rdy && app_wdf_rdy))
	                ||(state_cnt == READ && app_rdy)) ? 1'b1:1'b0;
	                
	//д״̬,MIGдЧʱдʹ
	assign app_wdf_wren = (state_cnt == WRITE && (app_rdy && app_wdf_rdy)) ? 1'b1:1'b0;
	
	//DDR3оƬʱӺûʱӵķƵѡ4:1ͻΪ8źͬ
	assign app_wdf_end = app_wdf_wren; 
	
	//ڶʱֵΪ1ʱֵΪ0
	assign app_cmd = (state_cnt == READ) ? 3'd1 :3'd0; 
	
	//ݶдַddrַ
	always @(*)  begin
	    if(~rst_n)
	        app_addr <= 0;
	    else if(state_cnt == READ )
	        if(ddr3_pingpang_en)
	            app_addr <= {2'b0,raddr_page,app_addr_rd[24:0]};
	        else 
	            app_addr <= {3'b0,app_addr_rd[24:0]};            
	    else if(ddr3_pingpang_en)
	        app_addr <= {2'b0,waddr_page,app_addr_wr[24:0]};
	    else
	        app_addr <= {3'b0,app_addr_wr[24:0]};        
	end  
	
	//źŽдĴ
	always @(posedge ui_clk or negedge rst_n)  begin
	    if(~rst_n)begin
	        rd_load_d0 <= 0;
	        rd_load_d1 <= 0; 
	        wr_load_d0 <= 0; 
	        wr_load_d1 <= 0;                    
	    end   
	    else begin
	        rd_load_d0 <= rd_load;
	        rd_load_d1 <= rd_load_d0;  
	        wr_load_d0 <= wr_load; 
	        wr_load_d1 <= wr_load_d0;                
	    end    
	end 
	
	//첽źŽдĴ
	always @(posedge ui_clk or negedge rst_n)  begin
	    if(~rst_n)begin
	        app_addr_rd_min_a <= 0;
	        app_addr_rd_max_a <= 0; 
	        rd_bust_len_a <= 0; 
	        app_addr_wr_min_a <= 0;  
	        app_addr_wr_max_a <= 0; 
	        wr_bust_len_a <= 0;                            
	    end   
	    else begin
	        app_addr_rd_min_a <= app_addr_rd_min;
	        app_addr_rd_max_a <= app_addr_rd_max; 
	        rd_bust_len_a <= rd_bust_len; 
	        app_addr_wr_min_a <= app_addr_wr_min;  
	        app_addr_wr_max_a <= app_addr_wr_max; 
	        wr_bust_len_a <= wr_bust_len;                    
	    end    
	end 
	 
	//Դ֡λ־
	always @(posedge ui_clk or negedge rst_n)  begin
	    if(~rst_n)
	        wr_rst <= 0;                
	    else if(wr_load_d0 && !wr_load_d1)
	        wr_rst <= 1;               
	    else
	        wr_rst <= 0;           
	end
	 
	//Դ֡λ־ 
	always @(posedge ui_clk or negedge rst_n)  begin
	    if(~rst_n)
	        rd_rst <= 0;                
	    else if(rd_load_d0 && !rd_load_d1)
	        rd_rst <= 1;               
	    else
	        rd_rst <= 0;           
	end
	
	//ԴĶַ֡λ 
	always @(posedge ui_clk or negedge rst_n)  begin
	    if(~rst_n)
	        raddr_rst_h <= 1'b0;
	    else if(rd_load_d0 && !rd_load_d1)
	        raddr_rst_h <= 1'b1;
	    else if(app_addr_rd == app_addr_rd_min_a)   
	        raddr_rst_h <= 1'b0;
	    else
	        raddr_rst_h <= raddr_rst_h;              
	end 
	
	//Դ֡λм 
	always @(posedge ui_clk or negedge rst_n)  begin
	    if(~rst_n)
	        raddr_rst_h_cnt <= 11'b0;
	    else if(raddr_rst_h)
	        raddr_rst_h_cnt <= raddr_rst_h_cnt + 1'b1;
	    else
	        raddr_rst_h_cnt <= 11'b0;            
	end 
	
	//Դ֡Ķַλл
	always @(posedge ui_clk or negedge rst_n)  begin
	    if(~rst_n)
	        raddr_page <= 1'b0;
	    else if( rd_end)
	        raddr_page <= ~waddr_page;         
	    else
	        raddr_page <= raddr_page;           
	end 
	  
	//Դ֡дַλл
	always @(posedge ui_clk or negedge rst_n)  begin
	    if(~rst_n)
	        waddr_page <= 1'b1;
	    else if( wr_end)
	        waddr_page <= ~waddr_page ;         
	    else
	        waddr_page <= waddr_page;           
	end   
	   
	//DDR3д߼ʵ
	always @(posedge ui_clk or negedge rst_n) begin
	    if(~rst_n) begin 
	        state_cnt    <= IDLE;              
	        wr_addr_cnt  <= 24'd0;      
	        rd_addr_cnt  <= 24'd0;       
	        app_addr_wr  <= 28'd0;   
	        app_addr_rd  <= 28'd0; 
	        wr_end       <= 1'b0;
	        rd_end       <= 1'b0;       
	    end
	    else begin
	        case(state_cnt)
	            IDLE:begin
	                if(init_calib_complete)
	                    state_cnt <= DDR3_DONE ;
	                else
	                    state_cnt <= IDLE;
	            end
	            DDR3_DONE:begin
	                if(wr_rst)begin   //֡λʱԼĴиλ
	                    state_cnt <= DDR3_DONE;
	                    wr_addr_cnt  <= 24'd0;	
	                    app_addr_wr <= app_addr_wr_min_a;					
				    end    //ַԼĴλ
	                else if(app_addr_rd >= app_addr_rd_max_a - 8)begin  
	                        state_cnt <= DDR3_DONE;
	                        rd_addr_cnt  <= 24'd0;      
	                        app_addr_rd <= app_addr_rd_min_a; 
	                        rd_end <= 1'b1;
	                end	   //дַԼĴλ
	                else if(app_addr_wr >= app_addr_wr_max_a - 8)begin  
	                        state_cnt <= DDR3_DONE;
	                        rd_addr_cnt  <= 24'd0;      
	                        app_addr_wr <= app_addr_wr_min_a; 
	                        wr_end <= 1'b1;
	                end	                		    
	                else if(wfifo_rcount >= wr_bust_len_a - 2 )begin  
	                    state_cnt <= WRITE;              //д
	                    wr_addr_cnt  <= 24'd0;                       
	                    app_addr_wr <= app_addr_wr;      //дֲַ
	                end
	                else if(raddr_rst_h)begin           //֡λʱԼĴиλ 
	                    if(raddr_rst_h_cnt >= 1000 && ddr3_read_valid)begin  
	                        state_cnt <= READ;         //֤fifoڸλʱд
	                        rd_addr_cnt  <= 24'd0;      
	                        app_addr_rd <= app_addr_rd_min_a; 
	                    end
	                    else begin
	                        state_cnt <= DDR3_DONE;
	                        rd_addr_cnt  <= 24'd0;      
	                        app_addr_rd <= app_addr_rd;                                
	                    end                                
	                end      //rfifo洢һͻʱ,ddrѾд1֡
	                else if(rfifo_wcount < rd_bust_len_a && ddr3_read_valid )begin  
	                    state_cnt <= READ;                              //
	                    rd_addr_cnt <= 24'd0;
	                    app_addr_rd <= app_addr_rd;      //ֲַ
	                end
	                else begin
	                    state_cnt <= state_cnt;   
	                    wr_addr_cnt  <= 24'd0;      
	                    rd_addr_cnt  <= 24'd0;  
	                    rd_end <= 1'b0;   
	                    wr_end <= 1'b0;                                       
	                end
	            end    
	            WRITE:   begin 
	                if((wr_addr_cnt == (wr_bust_len_a - 1)) && 
	                   (app_rdy && app_wdf_rdy))begin    //д趨ĳȴ״̬                  
	                    state_cnt    <= DDR3_DONE;       //д趨ĳȴ״̬               
	                    app_addr_wr <= app_addr_wr + 8;  //һд8ʼ8
	                end       
	                else if(app_rdy && app_wdf_rdy)begin   //д
	                    wr_addr_cnt  <= wr_addr_cnt + 1'd1;//дַԼ
	                    app_addr_wr  <= app_addr_wr + 8;   //һд8ʼ8
	                end
	                else begin                             //д㣬ֵǰֵ     
	                    wr_addr_cnt  <= wr_addr_cnt;
	                    app_addr_wr  <= app_addr_wr; 
	                end
	            end
	            READ:begin                      //趨ĵַ    
	                if((rd_addr_cnt == (rd_bust_len_a - 1)) && app_rdy)begin
	                    state_cnt   <= DDR3_DONE;          //״̬ 
	                    app_addr_rd <= app_addr_rd + 8;
	
	                end       
	                else if(app_rdy)begin               //MIGѾ׼,ʼ
	                    rd_addr_cnt <= rd_addr_cnt + 1'd1; //ûַÿμһ
	                    app_addr_rd <= app_addr_rd + 8; //һԶ8,DDR3ַ8
	                end
	                else begin                         //MIGû׼,򱣳ԭֵ
	                    rd_addr_cnt <= rd_addr_cnt;
	                    app_addr_rd <= app_addr_rd; 
	                end
	            end             
	            default:begin
	                state_cnt    <= IDLE;
	                wr_addr_cnt  <= 24'd0;
	                rd_addr_cnt  <= 24'd0;
	            end
	        endcase
	    end
	end 
	
	
	
endmodule    //ddr3_rw

