module digital_top(
	clk,
	rst_n,
	pre_frame_vsync,
	pre_frame_hsync,
	pre_frame_de,
	pre_rgb,
	xpos,
	ypos,
	h_total_pexel,
	v_total_pexel,
	post_frame_vsync,
	post_frame_hsync,
	post_frame_de,
	post_rgb,
	digit);

	//----Parameters:: generated by Robei-----
	parameter NUM_ROW = 1  ;
	parameter NUM_COL = 4  ;
	parameter DEPBIT  = 11 ;

	//---Ports declearation: generated by Robei---
	input clk;
	input rst_n;
	input pre_frame_vsync;
	input pre_frame_hsync;
	input pre_frame_de;
	input [15:0] pre_rgb;
	input [10:0] xpos;
	input [10:0] ypos;
	input [10:0] h_total_pexel;
	input [10:0] v_total_pexel;
	output post_frame_vsync;
	output post_frame_hsync;
	output post_frame_de;
	output [15:0] post_rgb;
	output [23:0] digit;

	wire clk;
	wire rst_n;
	wire pre_frame_vsync;
	wire pre_frame_hsync;
	wire pre_frame_de;
	wire [15:0] pre_rgb;
	wire [10:0] xpos;
	wire [10:0] ypos;
	wire [10:0] h_total_pexel;
	wire [10:0] v_total_pexel;
	wire post_frame_vsync;
	wire post_frame_hsync;
	wire post_frame_de;
	wire [15:0] post_rgb;
	wire [23:0] digit;
	wire rgb2ycbcr2_post_frame_vsync;
	wire rgb2ycbcr2_post_frame_hsync;
	wire rgb2ycbcr2_post_frame_de;
	wire [7:0] rgb2ycbcr2_img_y;
	wire binarization1_post_frame_vsync;
	wire binarization1_post_frame_hsync;
	wire binarization1_post_frame_de;
	wire binarization1_monoc;
	wire binarization1_monoc_fall;
	wire [4:0] change45_img_red;
	wire [5:0] change45_img_green;
	wire [4:0] change45_img_blue;
	wire [DEPBIT-1:0] projection3_row_border_data_rd;
	wire [DEPBIT-1:0] projection3_col_border_data_rd;
	wire [10:0] digital_recognition4_row_border_addr;
	wire [10:0] digital_recognition4_col_border_addr;
	wire [1:0] projection3_frame_cnt;
	wire projection3_project_done_flag;
	wire [3:0] projection3_num_col;
	wire [3:0] projection3_num_row;

	//----Code starts here: integrated by Robei-----
	
	
	
	
	//---Module instantiation---
	binarization binarization1(
		.clk(clk),
		.rst_n(rst_n),
		.pre_frame_vsync(rgb2ycbcr2_post_frame_vsync),
		.pre_frame_hsync(rgb2ycbcr2_post_frame_hsync),
		.pre_frame_de(rgb2ycbcr2_post_frame_de),
		.color(rgb2ycbcr2_img_y),
		.post_frame_vsync(binarization1_post_frame_vsync),
		.post_frame_hsync(binarization1_post_frame_hsync),
		.post_frame_de(binarization1_post_frame_de),
		.monoc(binarization1_monoc),
		.monoc_fall(binarization1_monoc_fall));

	rgb2ycbcr rgb2ycbcr2(
		.clk(clk),
		.rst_n(rst_n),
		.pre_frame_vsync(pre_frame_vsync),
		.pre_frame_hsync(pre_frame_hsync),
		.pre_frame_de(pre_frame_de),
		.img_red(change45_img_red),
		.img_green(change45_img_green),
		.img_blue(change45_img_blue),
		.post_frame_vsync(rgb2ycbcr2_post_frame_vsync),
		.post_frame_hsync(rgb2ycbcr2_post_frame_hsync),
		.post_frame_de(rgb2ycbcr2_post_frame_de),
		.img_y(rgb2ycbcr2_img_y));

	projection #(  1,  4, 1280, 800, 11) projection3(
		.clk(clk),
		.rst_n(rst_n),
		.frame_vsync(binarization1_post_frame_vsync),
		.frame_hsync(binarization1_post_frame_hsync),
		.frame_de(binarization1_post_frame_de),
		.monoc(binarization1_monoc),
		.xpos(xpos),
		.ypos(ypos),
		.row_border_addr_rd(digital_recognition4_row_border_addr),
		.col_border_addr_rd(digital_recognition4_col_border_addr),
		.h_total_pexel(h_total_pexel),
		.v_total_pexel(v_total_pexel),
		.row_border_data_rd(projection3_row_border_data_rd),
		.col_border_data_rd(projection3_col_border_data_rd),
		.num_col(projection3_num_col),
		.num_row(projection3_num_row),
		.frame_cnt(projection3_frame_cnt),
		.project_done_flag(projection3_project_done_flag));

	digital_recognition #(  1,  4, (NUM_ROW*NUM_COL<<2)-1) digital_recognition4(
		.clk(clk),
		.rst_n(rst_n),
		.monoc(binarization1_monoc),
		.monoc_fall(binarization1_monoc_fall),
		.xpos(xpos),
		.ypos(ypos),
		.row_border_data(projection3_row_border_data_rd),
		.col_border_data(projection3_col_border_data_rd),
		.frame_cnt(projection3_frame_cnt),
		.project_done_flag(projection3_project_done_flag),
		.num_col(projection3_num_col),
		.num_row(projection3_num_row),
		.row_border_addr(digital_recognition4_row_border_addr),
		.col_border_addr(digital_recognition4_col_border_addr),
		.color_rgb(post_rgb),
		.digit(digit));

	change4 change45(
		.pre_rgb(pre_rgb),
		.img_red(change45_img_red),
		.img_green(change45_img_green),
		.img_blue(change45_img_blue));

endmodule    //digital_top

